@@ -1,14 +1,23 @@
 from __future__ import division
+from __future__ import absolute_import
 import time
 import psutil
 from threading import Thread
 import numpy as np
 
 class MonitoringThread(Thread):
-    def __init__(self, aetros_backend, trainer):
+    def __init__(self, job_backend):
         Thread.__init__(self)
-        self.trainer = trainer
-        self.aetros_backend = aetros_backend
+
+        import sys
+        if 'theano.sandbox' in sys.modules:
+            # at this point, theano is already initialised, so we can use it to monitor the GPU.
+            from theano.sandbox import cuda
+            self.on_gpu = cuda.use.device_number is not None
+        else:
+            self.on_gpu = False
+
+        self.job_backend = job_backend
         self.second = 0
         self.running = True
 
@@ -18,7 +27,7 @@ class MonitoringThread(Thread):
     def run(self):
         while self.running:
             self.monitor()
-            time.sleep(0.1)
+            time.sleep(0.05)
 
     def monitor(self):
         cpu_util = np.mean(psutil.cpu_percent(interval=1, percpu=True))
@@ -26,13 +35,16 @@ class MonitoringThread(Thread):
 
         gpu_memory_use = None
 
-        if self.trainer.on_gpu:
+        if self.on_gpu:
             from theano.sandbox import cuda
             if cuda.cuda_ndarray.cuda_ndarray.mem_info:
                 gpu = cuda.cuda_ndarray.cuda_ndarray.mem_info()
-                gpu_memory_use = (gpu[1] - gpu[0]) / gpu[1] * 100
 
-        self.aetros_backend.job_add_status('system', {
+                if gpu[1] != 0:
+                    #sometimes, theano returns 0 as total memory, which will fail in next line
+                    gpu_memory_use = (gpu[1] - gpu[0]) / gpu[1] * 100
+
+        self.job_backend.job_add_status('system', {
             'second': self.second,
             'cpu': cpu_util,
             'memory': mem.percent,
@@ -40,4 +52,4 @@ class MonitoringThread(Thread):
         })
 
         self.second += 1
-        pass
\ No newline at end of file
+        pass
