@@ -1,26 +1,32 @@
+from __future__ import absolute_import
+from __future__ import print_function
 import sys
 import os
 from argparse import RawTextHelpFormatter
 
-import aetros.const
+from aetros import const
+from aetros.commands.ServerCommand import ServerCommand
 from aetros.commands.UploadWeightsCommand import UploadWeightsCommand
 from aetros.commands.PredictCommand import PredictCommand
-from aetros.commands.ServerCommand import ServerCommand
+from aetros.commands.PredictionServerCommand import PredictionServerCommand
 from aetros.commands.StartCommand import StartCommand
 
 commands_dict = {
     'start': StartCommand,
     'predict': PredictCommand,
     'upload-weights': UploadWeightsCommand,
-    'server': ServerCommand
+    'prediction-server': PredictionServerCommand,
+    'server': ServerCommand,
 }
 command_summaries = [
-    ['start', 'Starts a training of a network in current working directory'],
+    ['start', 'Starts a job of a model in current working directory'],
     ['predict', 'Runs a prediction locally'],
     ['upload-weights', 'Uploads weights as new or existing job.'],
-    ['server', 'Spawns a http server that handles incoming data as input and predicts output.'],
+    ['prediction-server', 'Spawns a http server that handles incoming data as input and predicts output.'],
+    ['server', 'Spawns a worker server that handles jobs managed through AETROS Trainer'],
 ]
 
+
 def create_main_parser():
     import argparse
     parser = argparse.ArgumentParser(formatter_class=RawTextHelpFormatter, add_help=False, prog=const.__prog__)
@@ -43,12 +49,13 @@ def create_main_parser():
 
     return parser
 
+
 def parseopts(args):
     if len(args) == 0:
 
         description = [''] + ['%-27s %s' % (i, j) for i, j in command_summaries]
         print("usage: API_KEY='key' aetros [command]\n")
-        print('Please don not forget to provide API_KEY as environment variable.\n\nPossible commands:\n' + ('\n'.join(description)))
+        print(('Please don not forget to provide API_KEY as environment variable.\n\nPossible commands:\n' + ('\n'.join(description))))
 
         sys.exit(1)
 
@@ -67,9 +74,9 @@ def main(args=None):
     cmd_name, cmd_args = parseopts(args)
 
     if cmd_name not in commands_dict:
-        print("Command %s not found" % (cmd_name, ))
+        print(("Command %s not found" % (cmd_name, )))
         sys.exit(1)
 
     command = commands_dict[cmd_name]()
 
-    return command.main(cmd_args)
\ No newline at end of file
+    return command.main(cmd_args)
