@@ -1,5 +1,5 @@
-__version__ = '0.3.3'
-__prog__ = "API_KEY='key' aetros"
+__version__ = '0.5.1'
+__prog__ = "aetros"
 
 class bcolors:
     HEADER = '\033[95m'
