@@ -1,4 +1,5 @@
 from __future__ import print_function
+from __future__ import absolute_import
 
 import io
 import json
@@ -6,35 +7,28 @@ import tempfile
 import urllib
 
 import numpy as np
-import sys
 import os
 
-from PIL import Image
+from aetros import keras_model_utils
+from aetros.keras_model_utils import ensure_dir
+from .backend import JobBackend, invalid_json_values
 
-from aetros import network
-from aetros.network import ensure_dir
-from JobModel import JobModel
-from AetrosBackend import AetrosBackend
+def predict(job_id, file_paths, insights=False, weights_path=None, api_token=None):
+    print("Prepare model ...")
+    job_backend = JobBackend(api_token=api_token)
+    job_backend.load(job_id)
 
-
-def predict(job_id, file_paths, insights=False, weights_path=None):
-    print("Prepare network ...")
-    aetros_backend = AetrosBackend(job_id)
-
-    job = aetros_backend.get_light_job()
-    job_id = job['id']
+    job_model = job_backend.get_job_model()
 
     log = io.open(tempfile.mktemp(), 'w', encoding='utf8')
     log.truncate()
 
-    network.job_prepare(job)
-
-    job_model = JobModel(aetros_backend, job)
+    keras_model_utils.job_prepare(job_model)
 
     if not weights_path:
         weight_path = job_model.get_weights_filepath_best()
         if not os.path.exists(weight_path) or os.path.getsize(weight_path) == 0:
-            weight_url = aetros_backend.get_best_weight_url(job_id)
+            weight_url = job_backend.get_best_weight_url(job_id)
             if not weight_url:
                 print("No weights available for this job.")
                 exit(1)
@@ -46,11 +40,11 @@ def predict(job_id, file_paths, insights=False, weights_path=None):
             f.write(urllib.urlopen(weight_url).read())
             f.close()
 
-    from GeneralLogger import GeneralLogger
-    from Trainer import Trainer
+    from .logger import GeneralLogger
+    from .Trainer import Trainer
 
-    general_logger = GeneralLogger(job, log, aetros_backend)
-    trainer = Trainer(aetros_backend, job_model, general_logger)
+    general_logger = GeneralLogger(log, job_backend)
+    trainer = Trainer(job_backend, general_logger)
     job_model.set_input_shape(trainer)
 
     print("Load model and compile ...")
@@ -66,8 +60,4 @@ def predict(job_id, file_paths, insights=False, weights_path=None):
     print("Start prediction ...")
 
     prediction = job_model.predict(model, np.array(inputs))
-    print(json.dumps(prediction, indent=4))
-
-
-
-
+    print(json.dumps(prediction, indent=4, default=invalid_json_values))
